"""
Wrapper Python pour l'API Atmo Data
API d'accès aux données de qualité de l'air des AASQA françaises
"""

import requests
from datetime import datetime, timedelta
from typing import Optional, Dict, Any, List, Union
import json
import re
import csv
import os
from pathlib import Path

from .constants import (
    FORMATS_VALIDES, AASQA_CODES, POLLUANTS, SECTEURS_EMISSIONS,
    INDICES_ATMO, INDICES_POLLENS, TYPES_EPISODES, ECHEANCES_VALIDES,
    ECHELLES_VALIDES, DEFAULT_API_URL, DEFAULT_TIMEOUT, TOKEN_VALIDITY_HOURS,
    SAVE_FORMATS, FILE_EXTENSIONS
)
from .models import AtmoDataCollection
from .exceptions import AtmoDataException


class AtmoDataClient:
    """Client pour l'API Atmo Data"""
    
    def __init__(self, base_url: Optional[str] = None, credentials_file: Optional[str] = None):
        self.credentials_file = credentials_file or "credentials.json"
        self._credentials = None
        
        # Déterminer l'URL de base : paramètre > credentials > défaut
        if base_url:
            self.base_url = base_url.rstrip('/')
        else:
            try:
                credentials = self._load_credentials()
                self.base_url = credentials.get('api_url', DEFAULT_API_URL).rstrip('/')
            except:
                self.base_url = DEFAULT_API_URL.rstrip('/')
        
        self.session = requests.Session()
        self.session.timeout = DEFAULT_TIMEOUT
        self.token = None
    
    def _validate_format(self, format_value: str) -> None:
        """Valide le format de sortie supportés par l'API"""
        if format_value not in FORMATS_VALIDES:
            raise ValueError(f"Format invalide: {format_value}. Formats valides: {FORMATS_VALIDES}")
    
    def _validate_aasqa(self, aasqa: str) -> None:
        """Valide le code AASQA"""
        if aasqa not in AASQA_CODES:
            raise ValueError(f"Code AASQA invalide: {aasqa}. Codes valides: {list(AASQA_CODES.keys())}")
    
    def _validate_polluant(self, polluant: str) -> None:
        """Valide le polluant"""
        if polluant not in POLLUANTS:
            raise ValueError(f"Polluant invalide: {polluant}. Polluants valides: {POLLUANTS}")
    
    def _validate_date(self, date: str) -> None:
        """Valide le format de date (YYYY-MM-DD)"""
        if not re.match(r'^\d{4}-\d{2}-\d{2}$', date):
            raise ValueError(f"Format de date invalide: {date}. Format attendu: YYYY-MM-DD")
        
        try:
            datetime.strptime(date, '%Y-%m-%d')
        except ValueError:
            raise ValueError(f"Date invalide: {date}")
    
    def _validate_code_qualificatif_atmo(self, code: str) -> None:
        """Valide le code qualificatif ATMO"""
        if code not in INDICES_ATMO:
            raise ValueError(f"Code qualificatif ATMO invalide: {code}. Codes valides: {list(INDICES_ATMO.keys())}")
    
    def _validate_code_qualificatif_pollen(self, code: str) -> None:
        """Valide le code qualificatif pollen"""
        if code not in INDICES_POLLENS:
            raise ValueError(f"Code qualificatif pollen invalide: {code}. Codes valides: {list(INDICES_POLLENS.keys())}")
    
    def _validate_type_episode(self, type_episode: str) -> None:
        """Valide le type d'épisode de pollution"""
        if type_episode not in TYPES_EPISODES:
            raise ValueError(f"Type d'épisode invalide: {type_episode}. Types valides: {TYPES_EPISODES}")
    
    def _validate_echeance(self, echeance: str) -> None:
        """Valide l'échéance"""
        if echeance not in ECHEANCES_VALIDES:
            raise ValueError(f"Échéance invalide: {echeance}. Échéances valides: {ECHEANCES_VALIDES}")
    
    def _validate_echelle(self, echelle: str) -> None:
        """Valide l'échelle"""
        if echelle not in ECHELLES_VALIDES:
            raise ValueError(f"Échelle invalide: {echelle}. Échelles valides: {ECHELLES_VALIDES}")
    
    def _validate_secteur(self, secteur: str) -> None:
        """Valide le secteur d'émission"""
        if secteur not in SECTEURS_EMISSIONS:
            raise ValueError(f"Secteur invalide: {secteur}. Secteurs valides: {list(SECTEURS_EMISSIONS.keys())}")
    
    def _validate_bounding_box(self, bounding_box: str) -> None:
        """Valide le format de la bounding box"""
        parts = bounding_box.split()
        if len(parts) != 4:
            raise ValueError(f"Format de bounding box invalide: {bounding_box}. Format attendu: 'xmin ymin xmax ymax'")
        
        try:
            coords = [float(part) for part in parts]
            if coords[0] >= coords[2] or coords[1] >= coords[3]:
                raise ValueError("Bounding box invalide: xmin doit être < xmax et ymin doit être < ymax")
        except ValueError as e:
            if "invalid literal for float()" in str(e):
                raise ValueError(f"Coordonnées de bounding box invalides: {bounding_box}")
            raise
    
    def _load_credentials(self) -> Dict[str, str]:
        """Charge les credentials depuis le fichier JSON"""
        if self._credentials is not None:
            return self._credentials
        
        try:
            with open(self.credentials_file, 'r', encoding='utf-8') as f:
                self._credentials = json.load(f)
                
            # Valider les champs requis
            required_fields = ['username', 'password']
            missing_fields = [field for field in required_fields if field not in self._credentials]
            
            if missing_fields:
                raise ValueError(f"Champs manquants dans {self.credentials_file}: {missing_fields}")
            
            # Mettre à jour l'URL de base si spécifiée dans les credentials
            if 'api_url' in self._credentials:
                self.base_url = self._credentials['api_url'].rstrip('/')
            
            return self._credentials
            
        except FileNotFoundError:
            raise AtmoDataException(
                f"Fichier de credentials '{self.credentials_file}' non trouvé. "
                f"Créez-le à partir de '{self.credentials_file}.example' ou utilisez login() avec username/password."
            )
        except json.JSONDecodeError as e:
            raise AtmoDataException(f"Erreur de format JSON dans {self.credentials_file}: {e}")
    
    def login(self, username: Optional[str] = None, password: Optional[str] = None) -> bool:
        """
        Connexion à l'API et récupération du token JWT
        
        Args:
            username: Nom d'utilisateur (optionnel si fichier credentials.json existe)
            password: Mot de passe (optionnel si fichier credentials.json existe)
            
        Returns:
            bool: True si la connexion réussit
        """
        # Si pas de username/password fournis, charger depuis le fichier
        if username is None or password is None:
            credentials = self._load_credentials()
            username = username or credentials['username']
            password = password or credentials['password']
        
        url = f"{self.base_url}/api/login"
        data = {
            "username": username,
            "password": password
        }
        
        try:
            response = self.session.post(url, json=data)
            response.raise_for_status()
            
            result = response.json()
            if 'token' in result:
                self.token = result['token']
                self.session.headers.update({
                    'Authorization': f'Bearer {self.token}'
                })
                return True
            return False
            
        except requests.exceptions.RequestException as e:
            raise AtmoDataException(f"Erreur de connexion: {e}")
    
    def auto_login(self) -> bool:
        """
        Connexion automatique en utilisant le fichier de credentials
        
        Returns:
            bool: True si la connexion réussit
        """
        return self.login()
    
    def _make_request(self, endpoint: str, params: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        Effectue une requête à l'API
        
        Args:
            endpoint: Endpoint de l'API
            params: Paramètres de la requête
            
        Returns:
            Dict: Réponse de l'API
        """
        if not self.token:
            raise AtmoDataException("Token non disponible. Veuillez vous connecter avec login()")
        
        url = f"{self.base_url}{endpoint}"
        
        try:
            response = self.session.get(url, params=params)
            response.raise_for_status()
            
            content_type = response.headers.get('content-type', '')
            if 'application/json' in content_type:
                return response.json()
            else:
                return {'data': response.text}
                
        except requests.exceptions.RequestException as e:
            raise AtmoDataException(f"Erreur lors de la requête: {e}")
    
    def get_indices_atmo(self, 
                        format: str = "geojson",
                        date: Optional[str] = None,
                        date_historique: Optional[str] = None,
                        code_zone: Optional[str] = None,
                        aasqa: Optional[str] = None,
                        code_qualificatif: Optional[str] = None,
                        bounding_box: Optional[str] = None,
                        bounding_box_srs: Optional[str] = None) -> Union[Dict[str, Any], AtmoDataCollection]:
        """
        Récupération des indices ATMO
        
        Args:
            format: Format de sortie ('geojson' ou 'csv')
            date: Date au format YYYY-MM-DD
            date_historique: Date de début pour une période
            code_zone: Code INSEE (commune/EPCI)
            aasqa: Code AASQA (01-94)
            code_qualificatif: Valeur d'indice (0-7)
            bounding_box: Bounding box "xmin ymin xmax ymax"
            bounding_box_srs: EPSG de la bounding box
            
        Returns:
            Union[Dict, AtmoDataCollection]: Données des indices ATMO (Dict pour CSV, AtmoDataCollection pour GeoJSON)
        """
        # Validation et construction des paramètres
        self._validate_format(format)
        params = {'format': format}
        
        if date:
            self._validate_date(date)
            params['date'] = date
        if date_historique:
            self._validate_date(date_historique)
            params['date_historique'] = date_historique
        if code_zone:
            params['code_zone'] = code_zone
        if aasqa:
            self._validate_aasqa(aasqa)
            params['aasqa'] = aasqa
        if code_qualificatif:
            self._validate_code_qualificatif_atmo(code_qualificatif)
            params['code_qualificatif'] = code_qualificatif
        if bounding_box:
            self._validate_bounding_box(bounding_box)
            params['bounding_box'] = bounding_box
        if bounding_box_srs:
            params['bounding_box_srs'] = bounding_box_srs
            
        result = self._make_request('/api/v2/data/indices/atmo', params)
        
        # Retourner un objet typé pour GeoJSON, dict brut pour CSV
        if format == 'geojson' and 'type' in result:
            return AtmoDataCollection(result, 'indices')
        return result
    
    def get_episodes_3jours(self,
                           format: str = "geojson",
                           aasqa: Optional[str] = None,
                           code_zone: Optional[str] = None,
                           polluant: Optional[str] = None,
                           type_episode: Optional[str] = None,
                           echeance: Optional[str] = None,
                           bounding_box: Optional[str] = None,
                           bounding_box_srs: Optional[str] = None) -> Union[Dict[str, Any], AtmoDataCollection]:
        """
        Récupération des épisodes de pollution sur 3 jours (J-1, J0, J+1)
        
        Args:
            format: Format de sortie ('geojson' ou 'csv')
            aasqa: Code AASQA (01-94)
            code_zone: Code INSEE (département ou bassin d'air)
            polluant: Polluant ('NO2', 'SO2', 'PM10', 'PM2.5', 'O3')
            type_episode: Type d'épisode
            echeance: Échéance ('-1', '0', '1')
            bounding_box: Bounding box "xmin ymin xmax ymax"
            bounding_box_srs: EPSG de la bounding box
            
        Returns:
            Union[Dict, AtmoDataCollection]: Données des épisodes (Dict pour CSV, AtmoDataCollection pour GeoJSON)
        """
        # Validation et construction des paramètres
        self._validate_format(format)
        params = {'format': format}
        
        if aasqa:
            self._validate_aasqa(aasqa)
            params['aasqa'] = aasqa
        if code_zone:
            params['code_zone'] = code_zone
        if polluant:
            self._validate_polluant(polluant)
            params['polluant'] = polluant
        if type_episode:
            self._validate_type_episode(type_episode)
            params['type_episode'] = type_episode
        if echeance:
            self._validate_echeance(echeance)
            params['echeance'] = echeance
        if bounding_box:
            self._validate_bounding_box(bounding_box)
            params['bounding_box'] = bounding_box
        if bounding_box_srs:
            params['bounding_box_srs'] = bounding_box_srs
            
        result = self._make_request('/api/v2/data/episodes/3jours', params)
        
        # Retourner un objet typé pour GeoJSON, dict brut pour CSV
        if format == 'geojson' and 'type' in result:
            return AtmoDataCollection(result, 'episodes')
        return result
    
    def get_episodes_historique(self,
                               date: str,
                               format: str = "geojson",
                               aasqa: Optional[str] = None,
                               code_zone: Optional[str] = None,
                               polluant: Optional[str] = None,
                               type_episode: Optional[str] = None,
                               date_historique: Optional[str] = None,
                               bounding_box: Optional[str] = None,
                               bounding_box_srs: Optional[str] = None) -> Union[Dict[str, Any], AtmoDataCollection]:
        """
        Récupération des épisodes de pollution historiques
        
        Args:
            date: Date de référence (obligatoire) au format YYYY-MM-DD
            format: Format de sortie ('geojson' ou 'csv')
            aasqa: Code AASQA (01-94)
            code_zone: Code INSEE (département ou bassin d'air)
            polluant: Polluant ('NO2', 'SO2', 'PM10', 'PM2.5', 'O3')
            type_episode: Type d'épisode
            date_historique: Date de début pour une période
            bounding_box: Bounding box "xmin ymin xmax ymax"
            bounding_box_srs: EPSG de la bounding box
            
        Returns:
            Union[Dict, AtmoDataCollection]: Données des épisodes historiques (Dict pour CSV, AtmoDataCollection pour GeoJSON)
        """
        # Validation et construction des paramètres
        self._validate_date(date)
        self._validate_format(format)
        params = {'date': date, 'format': format}
        
        if aasqa:
            self._validate_aasqa(aasqa)
            params['aasqa'] = aasqa
        if code_zone:
            params['code_zone'] = code_zone
        if polluant:
            self._validate_polluant(polluant)
            params['polluant'] = polluant
        if type_episode:
            self._validate_type_episode(type_episode)
            params['type_episode'] = type_episode
        if date_historique:
            self._validate_date(date_historique)
            params['date_historique'] = date_historique
        if bounding_box:
            self._validate_bounding_box(bounding_box)
            params['bounding_box'] = bounding_box
        if bounding_box_srs:
            params['bounding_box_srs'] = bounding_box_srs
            
        result = self._make_request('/api/v2/data/episodes/historique', params)
        
        # Retourner un objet typé pour GeoJSON, dict brut pour CSV
        if format == 'geojson' and 'type' in result:
            return AtmoDataCollection(result, 'episodes')
        return result
    
    def get_emissions(self,
                     format: str = "geojson",
                     echelle: str = "region",
                     code_zone: Optional[str] = None,
                     aasqa: Optional[str] = None,
                     secteur: Optional[str] = None,
                     bounding_box: Optional[str] = None,
                     bounding_box_srs: Optional[str] = None) -> Union[Dict[str, Any], AtmoDataCollection]:
        """
        Récupération des données d'inventaires des émissions
        
        Args:
            format: Format de sortie ('geojson' ou 'csv')
            echelle: Échelle d'agrégation ('region' ou 'epci')
            code_zone: Code zone (EPCI uniquement)
            aasqa: Code AASQA (01-94)
            secteur: Code secteur (5, 6, 7, 34, 219)
            bounding_box: Bounding box "xmin ymin xmax ymax"
            bounding_box_srs: EPSG de la bounding box
            
        Returns:
            Union[Dict, AtmoDataCollection]: Données des émissions (Dict pour CSV, AtmoDataCollection pour GeoJSON)
        """
        # Validation et construction des paramètres
        self._validate_format(format)
        self._validate_echelle(echelle)
        params = {'format': format, 'echelle': echelle}
        
        if code_zone:
            params['code_zone'] = code_zone
        if aasqa:
            self._validate_aasqa(aasqa)
            params['aasqa'] = aasqa
        if secteur:
            self._validate_secteur(secteur)
            params['secteur'] = secteur
        if bounding_box:
            self._validate_bounding_box(bounding_box)
            params['bounding_box'] = bounding_box
        if bounding_box_srs:
            params['bounding_box_srs'] = bounding_box_srs
            
        result = self._make_request('/api/v2/data/inventaires/emissions', params)
        
        # Retourner un objet typé pour GeoJSON, dict brut pour CSV
        if format == 'geojson' and 'type' in result:
            return AtmoDataCollection(result, 'emissions')
        return result
    
    def get_indices_pollens(self,
                           format: str = "geojson",
                           date: Optional[str] = None,
                           date_historique: Optional[str] = None,
                           code_zone: Optional[str] = None,
                           aasqa: Optional[str] = None,
                           code_qualificatif: Optional[str] = None,
                           alerte: Optional[bool] = None,
                           with_geom: bool = False,
                           bounding_box: Optional[str] = None,
                           bounding_box_srs: Optional[str] = None) -> Union[Dict[str, Any], AtmoDataCollection]:
        """
        Récupération des indices pollen
        
        Args:
            format: Format de sortie ('geojson' ou 'csv')
            date: Date au format YYYY-MM-DD
            date_historique: Date de début pour une période
            code_zone: Code INSEE (commune)
            aasqa: Code AASQA (01-94)
            code_qualificatif: Valeur d'indice pollinique (0-6)
            alerte: Filtre par statut d'alerte
            with_geom: Inclure les géométries
            bounding_box: Bounding box "xmin ymin xmax ymax"
            bounding_box_srs: EPSG de la bounding box
            
        Returns:
            Union[Dict, AtmoDataCollection]: Données des indices pollen (Dict pour CSV, AtmoDataCollection pour GeoJSON)
        """
        # Validation et construction des paramètres
        self._validate_format(format)
        params = {'format': format, 'with_geom': with_geom}
        
        if date:
            self._validate_date(date)
            params['date'] = date
        if date_historique:
            self._validate_date(date_historique)
            params['date_historique'] = date_historique
        if code_zone:
            params['code_zone'] = code_zone
        if aasqa:
            self._validate_aasqa(aasqa)
            params['aasqa'] = aasqa
        if code_qualificatif:
            self._validate_code_qualificatif_pollen(code_qualificatif)
            params['code_qualificatif'] = code_qualificatif
        if alerte is not None:
            params['alerte'] = alerte
        if bounding_box:
            self._validate_bounding_box(bounding_box)
            params['bounding_box'] = bounding_box
        if bounding_box_srs:
            params['bounding_box_srs'] = bounding_box_srs
            
        result = self._make_request('/api/v2/data/indices/pollens', params)
        
        # Retourner un objet typé pour GeoJSON, dict brut pour CSV
        if format == 'geojson' and 'type' in result:
            return AtmoDataCollection(result, 'pollens')
        return result
    
    def _validate_save_format(self, file_format: str) -> None:
        """Valide le format de sauvegarde"""
        if file_format not in SAVE_FORMATS:
            raise ValueError(f"Format de sauvegarde invalide: {file_format}. Formats supportés: {SAVE_FORMATS}")
    
    def _extract_features_to_csv_data(self, data: Dict[str, Any]) -> List[Dict[str, Any]]:
        """Extrait les features GeoJSON pour conversion en CSV"""
        csv_data = []
        
        if 'features' in data:
            for feature in data['features']:
                row = {}
                
                # Ajouter les propriétés
                if 'properties' in feature:
                    row.update(feature['properties'])
                
                # Ajouter les coordonnées si présentes
                if 'geometry' in feature and feature['geometry'] and 'coordinates' in feature['geometry']:
                    coords = feature['geometry']['coordinates']
                    if isinstance(coords, list) and len(coords) >= 2:
                        row['longitude'] = coords[0]
                        row['latitude'] = coords[1]
                
                csv_data.append(row)
        
        return csv_data
    
    def save_to_file(self, data: Dict[str, Any], filename: str, file_format: str = 'json') -> str:
        """
        Sauvegarde les données de l'API dans un fichier
        
        Args:
            data: Données retournées par l'API
            filename: Nom du fichier (sans extension)
            file_format: Format de sauvegarde ('json', 'csv', 'geojson')
            
        Returns:
            str: Chemin complet du fichier créé
            
        Raises:
            ValueError: Si le format n'est pas supporté
            IOError: Si erreur lors de l'écriture du fichier
        """
        self._validate_save_format(file_format)
        
        # Ajouter l'extension appropriée
        if not filename.endswith(FILE_EXTENSIONS[file_format]):
            filename += FILE_EXTENSIONS[file_format]
        
        # Créer le répertoire parent si nécessaire
        filepath = Path(filename)
        filepath.parent.mkdir(parents=True, exist_ok=True)
        
        try:
            if file_format == 'json':
                with open(filepath, 'w', encoding='utf-8') as f:
                    json.dump(data, f, indent=2, ensure_ascii=False)
            
            elif file_format == 'geojson':
                # Vérifier que c'est bien du GeoJSON
                if 'type' not in data or data['type'] != 'FeatureCollection':
                    raise ValueError("Les données ne sont pas au format GeoJSON")
                
                with open(filepath, 'w', encoding='utf-8') as f:
                    json.dump(data, f, indent=2, ensure_ascii=False)
            
            elif file_format == 'csv':
                csv_data = self._extract_features_to_csv_data(data)
                
                if not csv_data:
                    raise ValueError("Aucune donnée à convertir en CSV")
                
                # Obtenir tous les champs possibles
                fieldnames = set()
                for row in csv_data:
                    fieldnames.update(row.keys())
                fieldnames = sorted(fieldnames)
                
                with open(filepath, 'w', newline='', encoding='utf-8') as f:
                    writer = csv.DictWriter(f, fieldnames=fieldnames)
                    writer.writeheader()
                    writer.writerows(csv_data)
            
            return str(filepath.absolute())
            
        except ValueError:
            # Re-lancer les erreurs de validation sans les enrober
            raise
        except Exception as e:
            raise IOError(f"Erreur lors de la sauvegarde: {e}")


# AtmoDataException is now imported from .exceptions
